@@ -375,68 +375,16 @@ totem_embedded_set_state (TotemEmbedded *emb, TotemStates state)
 	emb->state = state;
 }
 
-static GdkPixbuf *
-totem_embedded_pad_pixbuf_for_size (GdkPixbuf *pixbuf,
-				    int width, int height)
-{
-	GdkPixbuf *logo;
-	guchar *pixels;
-	int rowstride, i;
-	int dest_x, dest_y;
-
-	logo = gdk_pixbuf_new (GDK_COLORSPACE_RGB,
-			       TRUE, 8, width, height);
-	pixels = gdk_pixbuf_get_pixels (logo);
-	rowstride = gdk_pixbuf_get_rowstride (logo);
-
-	/* Clear it */
-	for (i = 0; i < height; i++) {
-		memset (pixels + i * rowstride, 0, width * 4);
-	}
-
-	dest_x = (width - gdk_pixbuf_get_width (pixbuf)) / 2;
-	dest_y = (height - gdk_pixbuf_get_height (pixbuf)) / 2;
-	gdk_pixbuf_copy_area (pixbuf,
-			      0, 0,
-			      MIN (gdk_pixbuf_get_width (logo), gdk_pixbuf_get_width (pixbuf)),
-			      MIN (gdk_pixbuf_get_height (logo), gdk_pixbuf_get_height (pixbuf)),
-			      logo,
-			      dest_x,
-			      dest_y);
-
-	return logo;
-}
-
 static void
 totem_embedded_set_logo_by_name (TotemEmbedded *embedded,
 				 const char *name)
 {
-	GtkIconTheme *theme;
-	GdkPixbuf *logo, *padded;
-	int size, width, height;
-
 	totem_embedded_set_state (embedded, TOTEM_STATE_STOPPED);
 
 	if (embedded->audioonly != FALSE || embedded->hidden != FALSE)
 		return;
 
-	theme = gtk_icon_theme_get_for_screen (gtk_widget_get_screen (embedded->window));
-
-	width = GTK_WIDGET (embedded->bvw)->allocation.width;
-	height = GTK_WIDGET (embedded->bvw)->allocation.height;
-	size = MIN (width, height);
-
-	logo = gtk_icon_theme_load_icon (theme, name, size, 0, NULL);
-	if (logo == NULL) {
-		g_warning ("Couldn't load '%s' icon from theme", name);
-		return;
-	}
-	padded = totem_embedded_pad_pixbuf_for_size (logo, width, height);
-	g_object_unref (logo);
-	if (padded != NULL) {
-		bacon_video_widget_set_logo_pixbuf (embedded->bvw, padded);
-		bacon_video_widget_set_logo_mode (embedded->bvw, TRUE);
-	}
+	bacon_video_widget_set_logo (embedded->bvw, name);
 }
 
 static void
@@ -2206,7 +2154,7 @@ int main (int argc, char **argv)
 	if (XInitThreads () == 0)
 	{
 		gtk_init (&argc, &argv);
-		totem_embedded_error_and_exit (_("Could not initialise the thread-safe libraries."), _("Verify your system installation. The Totem plugin will now exit."), NULL);
+		totem_embedded_error_and_exit (_("Could not initialize the thread-safe libraries."), _("Verify your system installation. The Totem plugin will now exit."), NULL);
 	}
 
 	dbus_g_thread_init ();
@@ -2258,7 +2206,7 @@ int main (int argc, char **argv)
 	if (arg_audioonly != FALSE)
 		g_setenv("PULSE_PROP_media.role", "video", TRUE);
 	else
-		g_setenv("PULSE_PROP_media.role", "audio", TRUE);
+		g_setenv("PULSE_PROP_media.role", "music", TRUE);
 
         // FIXME check that ALL necessary params were given!
 	if (arg_plugin_type == TOTEM_PLUGIN_TYPE_LAST) {
@@ -2268,10 +2216,8 @@ int main (int argc, char **argv)
 
 	/* FIXME: check the UA strings of the legacy plugins themselves */
 	/* FIXME: at least hxplayer seems to send different UAs depending on the protocol!? */
-	/* FIXME: this won't work with gvfs:
-	 * http://bugzilla.gnome.org/show_bug.cgi?id=534482 */
 	if (arg_user_agent != NULL) {
-		g_setenv ("GNOME_VFS_HTTP_USER_AGENT", arg_user_agent, TRUE);
+		g_setenv ("BACON_VIDEO_WIDGET_HTTP_USER_AGENT", arg_user_agent, TRUE);
 		g_free (arg_user_agent);
 		arg_user_agent = NULL;
 	}
