@@ -35,6 +35,7 @@
 #include "totem-video-list.h"
 #include "totem-interface.h"
 #include "backend/bacon-video-widget.h"
+#include "totem-cell-renderer-video.h"
 
 /* Notebook pages */
 enum {
@@ -100,7 +101,7 @@ gboolean button_press_event_cb (GtkWidget *widget, GdkEventButton *event, TotemY
 gboolean button_release_event_cb (GtkWidget *widget, GdkEventButton *event, TotemYouTubePlugin *self);
 void open_in_web_browser_activate_cb (GtkAction *action, TotemYouTubePlugin *self);
 void value_changed_cb (GtkAdjustment *adjustment, TotemYouTubePlugin *self);
-gboolean starting_video_cb (TotemVideoList *video_list, GtkTreeView *tree_view, GtkTreePath *path, TotemYouTubePlugin *self);
+gboolean starting_video_cb (TotemVideoList *video_list, GtkTreePath *path, TotemYouTubePlugin *self);
 
 TOTEM_PLUGIN_REGISTER (TotemYouTubePlugin, totem_youtube_plugin)
 
@@ -144,7 +145,6 @@ new_from_stream_thread (GSimpleAsyncResult *result,
 	else
 		pixbuf = gdk_pixbuf_new_from_stream (stream, cancellable, &error);
 
-	g_free (data); /* GSimpleAsyncResult doesn't destroy result pointers when setting a new value over the top */
 	g_simple_async_result_set_op_res_gpointer (result, NULL, NULL);
 
 	/* Set the new pixbuf as the result, or error out */
@@ -278,6 +278,16 @@ set_up_tree_view (TotemYouTubePlugin *self, GtkBuilder *builder, guint key)
 	GtkActionGroup *action_group;
 	GtkAction *action, *menu_item;
 	GtkWidget *vscroll, *tree_view;
+	GtkTreeViewColumn *column;
+	GtkCellRenderer *renderer;
+
+	/* Add the cell renderer. This can't be done with GtkBuilder, because it unavoidably sets the expand parameter to FALSE */
+	/* TODO: Depends on bug #453692 */
+	renderer = GTK_CELL_RENDERER (totem_cell_renderer_video_new (TRUE));
+	column = GTK_TREE_VIEW_COLUMN (gtk_builder_get_object (builder,
+							       (key == SEARCH_TREE_VIEW) ? "yt_treeview_search_column" : "yt_treeview_related_column"));
+	gtk_tree_view_column_pack_start (column, renderer, TRUE);
+	gtk_tree_view_column_set_attributes (column, renderer, "thumbnail", 0, "title", 1, NULL);
 
 	/* Give the video lists a handle to Totem and connect their scrollbar signals */
 	if (key == SEARCH_TREE_VIEW) {
@@ -457,7 +467,8 @@ increment_progress_bar_fraction (TotemYouTubePlugin *self, guint tree_view)
 			gtk_widget_set_sensitive (self->cancel_button, FALSE);
 
 		/* Unref cancellable */
-		g_object_unref (self->cancellable[tree_view]);
+		if (self->cancellable[tree_view] != NULL)
+			g_object_unref (self->cancellable[tree_view]);
 		self->cancellable[tree_view] = NULL;
 	}
 }
@@ -521,9 +532,10 @@ resolve_t_param_cb (GObject *source_object, GAsyncResult *result, TParamData *da
 		GDataMediaContent *content;
 
 		/* We don't have a match, which is odd; fall back to the FLV URI as advertised by the YouTube API */
-		content = gdata_youtube_video_look_up_content (GDATA_YOUTUBE_VIDEO (data->entry), "application/x-shockwave-flash");
+		content = GDATA_MEDIA_CONTENT (gdata_youtube_video_look_up_content (GDATA_YOUTUBE_VIDEO (data->entry),
+										    "application/x-shockwave-flash"));
 		if (content != NULL) {
-			video_uri = g_strdup (content->uri);
+			video_uri = g_strdup (gdata_media_content_get_uri (content));
 			g_debug ("Couldn't find the t param of entry %s; falling back to its FLV URI (\"%s\")", video_id, video_uri);
 		} else {
 			/* Cop out */
@@ -557,7 +569,7 @@ resolve_t_param (TotemYouTubePlugin *self, GDataEntry *entry, GtkTreeIter *iter,
 	TParamData *data;
 
 	/* We have to get the t parameter from the actual HTML video page, since Google changed how their URIs work */
-	link = gdata_entry_look_up_link (entry, "alternate");
+	link = gdata_entry_look_up_link (entry, GDATA_LINK_ALTERNATE);
 	g_assert (link != NULL);
 
 	data = g_slice_new (TParamData);
@@ -566,7 +578,7 @@ resolve_t_param (TotemYouTubePlugin *self, GDataEntry *entry, GtkTreeIter *iter,
 	data->iter = *iter;
 	data->tree_view = tree_view;
 
-	video_page = g_file_new_for_uri (link->href);
+	video_page = g_file_new_for_uri (gdata_link_get_uri (link));
 	g_file_load_contents_async (video_page, self->cancellable[tree_view], (GAsyncReadyCallback) resolve_t_param_cb, data);
 	g_object_unref (video_page);
 }
@@ -588,18 +600,14 @@ thumbnail_loaded_cb (GObject *source_object, GAsyncResult *result, ThumbnailData
 	thumbnail = totem_gdk_pixbuf_new_from_stream_finish (result, &error);
 
 	if (thumbnail == NULL) {
-		GtkWindow *window;
-
 		/* Bail out if the operation was cancelled */
 		if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED) == TRUE) {
 			g_error_free (error);
 			goto free_data;
 		}
 
-		/* Display an error message */
-		window = totem_get_main_window (data->plugin->totem);
-		totem_interface_error (_("Error Loading Video Thumbnail"), error->message, window);
-		g_object_unref (window);
+		/* Don't display an error message, since this isn't really meant to happen */
+		g_warning ("Error loading video thumbnail: %s", error->message);
 		g_error_free (error);
 		goto free_data;
 	}
@@ -633,19 +641,15 @@ thumbnail_opened_cb (GObject *source_object, GAsyncResult *result, ThumbnailData
 	input_stream = g_file_read_finish (thumbnail_file, result, &error);
 
 	if (input_stream == NULL) {
-		GtkWindow *window;
-
-		/* Display an error message */
-		window = totem_get_main_window (data->plugin->totem);
-		totem_interface_error (_("Error Loading Video Thumbnail"), error->message, window);
-		g_object_unref (window);
+		/* Don't display an error message, since this isn't really meant to happen */
+		g_warning ("Error loading video thumbnail: %s", error->message);
 		g_error_free (error);
 		return;
 	}
 
 	g_debug ("Creating thumbnail from stream");
 	totem_gdk_pixbuf_new_from_stream_at_scale_async (G_INPUT_STREAM (input_stream), THUMBNAIL_WIDTH, -1, TRUE,
-						   self->cancellable[data->tree_view], (GAsyncReadyCallback) thumbnail_loaded_cb, data);
+							 self->cancellable[data->tree_view], (GAsyncReadyCallback) thumbnail_loaded_cb, data);
 	g_object_unref (input_stream);
 }
 
@@ -679,7 +683,17 @@ query_finished_cb (GObject *source_object, GAsyncResult *result, QueryData *data
 
 		/* Error! */
 		window = totem_get_main_window (data->plugin->totem);
-		totem_interface_error (_("Error Searching for Videos"), error->message, window);
+		if (g_error_matches (error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_PROTOCOL_ERROR) == TRUE) {
+			/* Hide the ugly technical message libgdata gives behind a nice one telling them it's out of date (which it likely is
+			 * if we're receiving a protocol error). */
+			totem_interface_error (_("Error Searching for Videos"),
+					       _("The response from the server could not be understood. "
+					         "Please check you are running the latest version of libgdata."), window);
+		} else {
+			/* Spew out the error message as provided */
+			totem_interface_error (_("Error Searching for Videos"), error->message, window);
+		}
+
 		g_object_unref (window);
 		g_error_free (error);
 		goto free_data;
@@ -738,9 +752,9 @@ query_progress_cb (GDataEntry *entry, guint entry_key, guint entry_count, QueryD
 		gint new_delta;
 		GDataMediaThumbnail *current_thumb = (GDataMediaThumbnail*) thumbnails->data;
 
-		g_debug ("%u pixel wide thumbnail available for entry %s", current_thumb->width, id);
+		g_debug ("%u pixel wide thumbnail available for entry %s", gdata_media_thumbnail_get_width (current_thumb), id);
 
-		new_delta = current_thumb->width - THUMBNAIL_WIDTH;
+		new_delta = gdata_media_thumbnail_get_width (current_thumb) - THUMBNAIL_WIDTH;
 		if (delta == 0) {
 			break;
 		} else if ((delta == G_MININT) ||
@@ -748,7 +762,8 @@ query_progress_cb (GDataEntry *entry, guint entry_key, guint entry_count, QueryD
 			   (delta > 0 && new_delta > 0 && new_delta < delta)) {
 			delta = new_delta;
 			thumbnail = current_thumb;
-			g_debug ("Choosing a %u pixel wide thumbnail (delta: %i) for entry %s", current_thumb->width, new_delta, id);
+			g_debug ("Choosing a %u pixel wide thumbnail (delta: %i) for entry %s",
+				 gdata_media_thumbnail_get_width (current_thumb), new_delta, id);
 		}
 	}
 
@@ -762,7 +777,7 @@ query_progress_cb (GDataEntry *entry, guint entry_key, guint entry_count, QueryD
 		t_data->tree_view = data->tree_view;
 
 		g_debug ("Starting thumbnail download for entry %s", id);
-		thumbnail_file = g_file_new_for_uri (thumbnail->uri);
+		thumbnail_file = g_file_new_for_uri (gdata_media_thumbnail_get_uri (thumbnail));
 		g_file_read_async (thumbnail_file, G_PRIORITY_DEFAULT, self->cancellable[data->tree_view],
 				   (GAsyncReadyCallback) thumbnail_opened_cb, t_data);
 		g_object_unref (thumbnail_file);
@@ -775,8 +790,10 @@ execute_query (TotemYouTubePlugin *self, guint tree_view, gboolean clear_tree_vi
 	QueryData *data;
 
 	/* Cancel previous searches on this tree view */
-	if (self->cancellable[tree_view] != NULL)
+	if (self->cancellable[tree_view] != NULL) {
 		g_cancellable_cancel (self->cancellable[tree_view]);
+		g_object_unref (self->cancellable[tree_view]);
+	}
 
 	/* Clear the tree views */
 	if (clear_tree_view == TRUE)
@@ -912,11 +929,11 @@ open_in_web_browser_activate_cb (GtkAction *action, TotemYouTubePlugin *self)
 
 		/* Get the HTML page for the video; its <link rel="alternate" ... /> */
 		gtk_tree_model_get (model, &iter, 3, &video, -1);
-		link = gdata_entry_look_up_link (GDATA_ENTRY (video), "alternate");
+		link = gdata_entry_look_up_link (GDATA_ENTRY (video), GDATA_LINK_ALTERNATE);
 		g_object_unref (video);
 
 		/* Display the page */
-		if (gtk_show_uri (gtk_widget_get_screen (GTK_WIDGET (self->bvw)), link->href, GDK_CURRENT_TIME, &error) == FALSE) {
+		if (gtk_show_uri (gtk_widget_get_screen (GTK_WIDGET (self->bvw)), gdata_link_get_uri (link), GDK_CURRENT_TIME, &error) == FALSE) {
 			GtkWindow *window = totem_get_main_window (self->totem);
 			totem_interface_error (_("Error Opening Video in Web Browser"), error->message, window);
 			g_object_unref (window);
@@ -960,7 +977,7 @@ button_release_event_cb (GtkWidget *widget, GdkEventButton *event, TotemYouTubeP
 }
 
 gboolean
-starting_video_cb (TotemVideoList *video_list, GtkTreeView *tree_view, GtkTreePath *path, TotemYouTubePlugin *self)
+starting_video_cb (TotemVideoList *video_list, GtkTreePath *path, TotemYouTubePlugin *self)
 {
 	GtkTreeIter iter;
 	GDataYouTubeVideo *video_entry;
