@@ -31,49 +31,53 @@
 #include "totem-interface.h"
 #include "totem-private.h"
 #include "totem-sidebar.h"
+#include "totem-statusbar.h"
 #include "totem-plugin-manager.h"
 #include "bacon-video-widget.h"
+#include "totem-uri.h"
 
-#include "debug.h"
+#include "totem-profile.h"
+
+#define TOTEM_MAX_RECENT_ITEM_LEN 40
 
 /* Callback functions for GtkBuilder */
-void open_action_callback (GtkAction *action, Totem *totem);
-void open_location_action_callback (GtkAction *action, Totem *totem);
-void eject_action_callback (GtkAction *action, Totem *totem);
-void properties_action_callback (GtkAction *action, Totem *totem);
-void play_action_callback (GtkAction *action, Totem *totem);
-void quit_action_callback (GtkAction *action, Totem *totem);
-void preferences_action_callback (GtkAction *action, Totem *totem);
-void fullscreen_action_callback (GtkAction *action, Totem *totem);
-void zoom_1_2_action_callback (GtkAction *action, Totem *totem);
-void zoom_1_1_action_callback (GtkAction *action, Totem *totem);
-void zoom_2_1_action_callback (GtkAction *action, Totem *totem);
-void zoom_in_action_callback (GtkAction *action, Totem *totem);
-void zoom_reset_action_callback (GtkAction *action, Totem *totem);
-void zoom_out_action_callback (GtkAction *action, Totem *totem);
-void next_angle_action_callback (GtkAction *action, Totem *totem);
-void dvd_root_menu_action_callback (GtkAction *action, Totem *totem);
-void dvd_title_menu_action_callback (GtkAction *action, Totem *totem);
-void dvd_audio_menu_action_callback (GtkAction *action, Totem *totem);
-void dvd_angle_menu_action_callback (GtkAction *action, Totem *totem);
-void dvd_chapter_menu_action_callback (GtkAction *action, Totem *totem);
-void next_chapter_action_callback (GtkAction *action, Totem *totem);
-void previous_chapter_action_callback (GtkAction *action, Totem *totem);
-void skip_forward_action_callback (GtkAction *action, Totem *totem);
-void skip_backwards_action_callback (GtkAction *action, Totem *totem);
-void volume_up_action_callback (GtkAction *action, Totem *totem);
-void volume_down_action_callback (GtkAction *action, Totem *totem);
-void contents_action_callback (GtkAction *action, Totem *totem);
-void about_action_callback (GtkAction *action, Totem *totem);
-void plugins_action_callback (GtkAction *action, Totem *totem);
-void repeat_mode_action_callback (GtkToggleAction *action, Totem *totem);
-void shuffle_mode_action_callback (GtkToggleAction *action, Totem *totem);
-void deinterlace_action_callback (GtkToggleAction *action, Totem *totem);
-void show_controls_action_callback (GtkToggleAction *action, Totem *totem);
-void show_sidebar_action_callback (GtkToggleAction *action, Totem *totem);
-void aspect_ratio_changed_callback (GtkRadioAction *action, GtkRadioAction *current, Totem *totem);
-void select_subtitle_action_callback (GtkAction *action, Totem *totem);
-void clear_playlist_action_callback (GtkAction *action, Totem *totem);
+G_MODULE_EXPORT void open_action_callback (GtkAction *action, Totem *totem);
+G_MODULE_EXPORT void open_location_action_callback (GtkAction *action, Totem *totem);
+G_MODULE_EXPORT void eject_action_callback (GtkAction *action, Totem *totem);
+G_MODULE_EXPORT void properties_action_callback (GtkAction *action, Totem *totem);
+G_MODULE_EXPORT void play_action_callback (GtkAction *action, Totem *totem);
+G_MODULE_EXPORT void quit_action_callback (GtkAction *action, Totem *totem);
+G_MODULE_EXPORT void preferences_action_callback (GtkAction *action, Totem *totem);
+G_MODULE_EXPORT void fullscreen_action_callback (GtkAction *action, Totem *totem);
+G_MODULE_EXPORT void zoom_1_2_action_callback (GtkAction *action, Totem *totem);
+G_MODULE_EXPORT void zoom_1_1_action_callback (GtkAction *action, Totem *totem);
+G_MODULE_EXPORT void zoom_2_1_action_callback (GtkAction *action, Totem *totem);
+G_MODULE_EXPORT void zoom_in_action_callback (GtkAction *action, Totem *totem);
+G_MODULE_EXPORT void zoom_reset_action_callback (GtkAction *action, Totem *totem);
+G_MODULE_EXPORT void zoom_out_action_callback (GtkAction *action, Totem *totem);
+G_MODULE_EXPORT void next_angle_action_callback (GtkAction *action, Totem *totem);
+G_MODULE_EXPORT void dvd_root_menu_action_callback (GtkAction *action, Totem *totem);
+G_MODULE_EXPORT void dvd_title_menu_action_callback (GtkAction *action, Totem *totem);
+G_MODULE_EXPORT void dvd_audio_menu_action_callback (GtkAction *action, Totem *totem);
+G_MODULE_EXPORT void dvd_angle_menu_action_callback (GtkAction *action, Totem *totem);
+G_MODULE_EXPORT void dvd_chapter_menu_action_callback (GtkAction *action, Totem *totem);
+G_MODULE_EXPORT void next_chapter_action_callback (GtkAction *action, Totem *totem);
+G_MODULE_EXPORT void previous_chapter_action_callback (GtkAction *action, Totem *totem);
+G_MODULE_EXPORT void skip_forward_action_callback (GtkAction *action, Totem *totem);
+G_MODULE_EXPORT void skip_backwards_action_callback (GtkAction *action, Totem *totem);
+G_MODULE_EXPORT void volume_up_action_callback (GtkAction *action, Totem *totem);
+G_MODULE_EXPORT void volume_down_action_callback (GtkAction *action, Totem *totem);
+G_MODULE_EXPORT void contents_action_callback (GtkAction *action, Totem *totem);
+G_MODULE_EXPORT void about_action_callback (GtkAction *action, Totem *totem);
+G_MODULE_EXPORT void plugins_action_callback (GtkAction *action, Totem *totem);
+G_MODULE_EXPORT void repeat_mode_action_callback (GtkToggleAction *action, Totem *totem);
+G_MODULE_EXPORT void shuffle_mode_action_callback (GtkToggleAction *action, Totem *totem);
+G_MODULE_EXPORT void deinterlace_action_callback (GtkToggleAction *action, Totem *totem);
+G_MODULE_EXPORT void show_controls_action_callback (GtkToggleAction *action, Totem *totem);
+G_MODULE_EXPORT void show_sidebar_action_callback (GtkToggleAction *action, Totem *totem);
+G_MODULE_EXPORT void aspect_ratio_changed_callback (GtkRadioAction *action, GtkRadioAction *current, Totem *totem);
+G_MODULE_EXPORT void select_subtitle_action_callback (GtkAction *action, Totem *totem);
+G_MODULE_EXPORT void clear_playlist_action_callback (GtkAction *action, Totem *totem);
 
 /* Helper function to escape underscores in labels
  * before putting them in menu items */
@@ -277,7 +281,7 @@ languages_changed_callback (GtkRadioAction *action, GtkRadioAction *current,
 static GtkAction *
 add_lang_action (Totem *totem, GtkActionGroup *action_group, guint ui_id,
 		const char **paths, const char *prefix, const char *lang, 
-		int lang_id, int index, GSList **group)
+		int lang_id, int lang_index, GSList **group)
 {
 	const char *full_lang;
 	char *label;
@@ -287,12 +291,12 @@ add_lang_action (Totem *totem, GtkActionGroup *action_group, guint ui_id,
 
 	full_lang = totem_lang_get_full (lang);
 
-	if (index > 1) {
+	if (lang_index > 1) {
 		char *num_lang;
 
 		num_lang = g_strdup_printf ("%s #%u",
 					    full_lang ? full_lang : lang,
-					    index);
+					    lang_index);
 		label = escape_label_for_menu (num_lang);
 		g_free (num_lang);
 	} else {
@@ -459,8 +463,7 @@ totem_subtitles_update (Totem *totem, GList *list)
 			totem->subtitles_action_group, -1);
 
 
-	if (list != NULL)
-	{
+	if (list != NULL) {
 		action = create_lang_actions (totem, totem->subtitles_action_group,
 				totem->subtitles_ui_id,
 				paths,
@@ -541,6 +544,7 @@ connect_proxy_cb (GtkActionGroup *action_group,
         }
 
         gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (proxy), image);
+        gtk_image_menu_item_set_always_show_image (GTK_IMAGE_MENU_ITEM (proxy), TRUE);
 }
 
 static void
@@ -694,6 +698,9 @@ totem_action_add_recent (Totem *totem, const char *uri)
 
 	memset (&data, 0, sizeof (data));
 
+	if (totem_is_special_mrl (uri) != FALSE)
+		return;
+
 	file = g_file_new_for_uri (uri);
 	file_info = g_file_query_info (file,
 				       G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE "," G_FILE_ATTRIBUTE_STANDARD_DISPLAY_NAME,
@@ -704,13 +711,13 @@ totem_action_add_recent (Totem *totem, const char *uri)
 		data.display_name = NULL;
 		/* Bogus mime-type, we just want it added */
 		data.mime_type = g_strdup ("video/x-totem-stream");
-		groups[0] = "TotemStreams";
+		groups[0] = (gchar*) "TotemStreams";
 		g_message ("no file info");
 	} else {
 		data.mime_type = g_strdup (g_file_info_get_content_type (file_info));
 		data.display_name = g_strdup (g_file_info_get_display_name (file_info));
 		g_object_unref (file_info);
-		groups[0] = "Totem";
+		groups[0] = (gchar*) "Totem";
 	}
 	g_object_unref (file);
 
@@ -772,6 +779,8 @@ add_drive_to_menu (GDrive *drive, guint position, Totem *totem)
 		const char *icon_name;
 		guint j;
 		char *device_path;
+		GtkWidget *menu_item;
+		char *menu_item_path;
 
 		disabled = FALSE;
 
@@ -841,6 +850,14 @@ add_drive_to_menu (GDrive *drive, guint position, Totem *totem)
 			"/tmw-menubar/movie/devices-placeholder", name, name,
 			GTK_UI_MANAGER_MENUITEM, FALSE);
 
+		/* TODO: This can be made cleaner once bug #589842 is fixed */
+		menu_item_path = g_strdup_printf ("/tmw-menubar/movie/devices-placeholder/%s", name);
+		menu_item = gtk_ui_manager_get_widget (totem->ui_manager, menu_item_path);
+		g_free (menu_item_path);
+
+		if (menu_item != NULL)
+			gtk_image_menu_item_set_always_show_image (GTK_IMAGE_MENU_ITEM (menu_item), TRUE);
+
 		g_free (name);
 		g_free (label);
 		g_object_unref (icon);
@@ -1170,14 +1187,11 @@ about_action_callback (GtkAction *action, Totem *totem)
 		"Ronald Bultje <rbultje@ronald.bitfreak.net>",
 		"Julien Moutte <julien@moutte.net> (GStreamer backend)",
 		"Tim-Philipp M\303\274ller <tim\100centricular\056net> (GStreamer backend)",
+		"Philip Withnall <philip@tecnocode.co.uk>",
 		NULL
 	};
 	const char *artists[] = { "Jakub Steiner <jimmac@ximian.com>", NULL };
-	const char *documenters[] =
-	{
-		"Chee Bin Hoh <cbhoh@gnome.org>",
-		NULL
-	};
+	#include "../help/totem-docs.h"
 	char *license = totem_interface_get_license ();
 
 	backend_version = bacon_video_widget_get_backend_name (totem->bvw);
@@ -1187,17 +1201,17 @@ about_action_callback (GtkAction *action, Totem *totem)
 
 	gtk_show_about_dialog (GTK_WINDOW (totem->win),
 				     "version", VERSION,
-				     "copyright", _("Copyright \xc2\xa9 2002-2008 Bastien Nocera"),
+				     "copyright", _("Copyright \xc2\xa9 2002-2009 Bastien Nocera"),
 				     "comments", description,
 				     "authors", authors,
-				     "documenters", documenters,
+				     "documenters", documentation_credits,
 				     "artists", artists,
 				     "translator-credits", _("translator-credits"),
 				     "logo-icon-name", "totem",
 				     "license", license,
 				     "wrap-license", TRUE,
 				     "website-label", _("Totem Website"),
-				     "website", "http://www.gnome.org/projects/totem/",
+				     "website", "http://projects.gnome.org/totem/",
 				     NULL);
 
 	g_free (backend_version);
@@ -1322,6 +1336,56 @@ clear_playlist_action_callback (GtkAction *action, Totem *totem)
 	totem_action_set_mrl (totem, NULL, NULL);
 }
 
+/* Show help in status bar when selecting (hovering over) a menu item. */
+static void
+menu_item_select_cb (GtkMenuItem *proxy, Totem *totem)
+{
+	GtkAction *action;
+	const gchar *message;
+
+	action = gtk_activatable_get_related_action (GTK_ACTIVATABLE (proxy));
+	g_return_if_fail (action != NULL);
+
+	message = gtk_action_get_tooltip (action);
+	if (message)
+		totem_statusbar_push_help (TOTEM_STATUSBAR (totem->statusbar), message);
+}
+
+static void
+menu_item_deselect_cb (GtkMenuItem *proxy, Totem *totem)
+{
+	totem_statusbar_pop_help (TOTEM_STATUSBAR (totem->statusbar));
+}
+
+static void
+setup_action (Totem *totem, GtkAction *action)
+{
+	GSList *proxies;
+	for (proxies = gtk_action_get_proxies (action); proxies != NULL; proxies = proxies->next) {
+		if (GTK_IS_MENU_ITEM (proxies->data)) {
+			g_signal_connect (proxies->data, "select", G_CALLBACK (menu_item_select_cb), totem);
+			g_signal_connect (proxies->data, "deselect", G_CALLBACK (menu_item_deselect_cb), totem);
+		}
+
+	}
+}
+
+static void
+setup_menu_items (Totem *totem)
+{
+	GList *action_groups;
+
+	/* FIXME: We can remove this once GTK+ bug #574001 is fixed */
+	for (action_groups = gtk_ui_manager_get_action_groups (totem->ui_manager);
+	     action_groups != NULL; action_groups = action_groups->next) {
+		GtkActionGroup *action_group = GTK_ACTION_GROUP (action_groups->data);
+		GList *actions;
+		for (actions = gtk_action_group_list_actions (action_group); actions != NULL; actions = actions->next) {
+			setup_action (totem, GTK_ACTION (actions->data));
+		}
+	}
+}
+
 static const GtkActionEntry seek_entries_ltr[] = {
 	{ "skip-forward", GTK_STOCK_MEDIA_FORWARD, N_("Skip _Forward"), "Right", N_("Skip forward"), G_CALLBACK (skip_forward_action_callback) },
 	{ "skip-backwards", GTK_STOCK_MEDIA_REWIND, N_("Skip _Backwards"), "Left", N_("Skip backwards"), G_CALLBACK (skip_backwards_action_callback) }
@@ -1369,6 +1433,8 @@ totem_ui_manager_setup (Totem *totem)
 
 	totem->ui_manager = GTK_UI_MANAGER (gtk_builder_get_object (totem->xml, "totem-ui-manager"));
 
+	setup_menu_items (totem);
+
 	totem->devices_action_group = NULL;
 	totem->devices_ui_id = gtk_ui_manager_new_merge_id (totem->ui_manager);
 	totem->languages_action_group = NULL;
