@@ -37,11 +37,10 @@
 #include "totem-uri.h"
 #include "totem-interface.h"
 #include "video-utils.h"
-#include "debug.h"
 
 #define PL_LEN (gtk_tree_model_iter_n_children (playlist->priv->model, NULL))
 
-static void ensure_shuffled (TotemPlaylist *playlist, gboolean shuffle);
+static void ensure_shuffled (TotemPlaylist *playlist);
 static gboolean totem_playlist_add_one_mrl (TotemPlaylist *playlist, const char *mrl, const char *display_name);
 
 typedef gboolean (*ClearComparisonFunc) (TotemPlaylist *playlist, GtkTreeIter *iter, gconstpointer data);
@@ -262,7 +261,7 @@ totem_playlist_error (char *title, char *reason, TotemPlaylist *playlist)
 void
 totem_playlist_select_subtitle_dialog(TotemPlaylist *playlist, TotemPlaylistSelectDialog mode)
 {
-	char *subtitle, *current, *path;
+	char *subtitle, *current, *uri;
 	GFile *file, *dir;
 	TotemPlaylistStatus playing;
 	GtkTreeIter iter;
@@ -284,23 +283,23 @@ totem_playlist_select_subtitle_dialog(TotemPlaylist *playlist, TotemPlaylistSele
 
 	/* Look for the directory of the current movie */
 	gtk_tree_model_get (playlist->priv->model, &iter,
-			    FILENAME_COL, &current,
+			    URI_COL, &current,
 			    -1);
 
 	if (current == NULL)
 		return;
 
-	path = NULL;
-	file = g_file_new_for_commandline_arg (current);
+	uri = NULL;
+	file = g_file_new_for_uri (current);
 	dir = g_file_get_parent (file);
 	g_object_unref (file);
 	if (dir != NULL) {
-		path = g_file_get_path (dir);
+		uri = g_file_get_uri (dir);
 		g_object_unref (dir);
 	}
 
-	subtitle = totem_add_subtitle (totem_playlist_get_toplevel (playlist), path);
-	g_free (path);
+	subtitle = totem_add_subtitle (totem_playlist_get_toplevel (playlist), uri);
+	g_free (uri);
 
 	if (subtitle == NULL)
 		return;
@@ -1380,24 +1379,23 @@ compare_random (gconstpointer ptr_a, gconstpointer ptr_b)
 }
 
 static void
-ensure_shuffled (TotemPlaylist *playlist, gboolean shuffle)
+ensure_shuffled (TotemPlaylist *playlist)
 {
 	RandomData data;
 	GArray *array;
 	int i, current, current_new;
 	int *indices;
 
-	if (shuffle == FALSE || PL_LEN != playlist->priv->shuffle_len)
-	{
-		g_free (playlist->priv->shuffled);
-		playlist->priv->shuffled = NULL;
-	}
+	if (playlist->priv->shuffled == NULL)
+		playlist->priv->shuffled = g_new (int, PL_LEN);
+	else if (PL_LEN != playlist->priv->shuffle_len)
+		playlist->priv->shuffled = g_renew (int, playlist->priv->shuffled, PL_LEN);
+	playlist->priv->shuffle_len = PL_LEN;
 
-	if (shuffle == FALSE || PL_LEN == 0)
+	if (PL_LEN == 0)
 		return;
 
-	if (playlist->priv->current != NULL)
-	{
+	if (playlist->priv->current != NULL) {
 		indices = gtk_tree_path_get_indices (playlist->priv->current);
 		current = indices[0];
 	} else {
@@ -1406,14 +1404,9 @@ ensure_shuffled (TotemPlaylist *playlist, gboolean shuffle)
 	
 	current_new = -1;
 
-	playlist->priv->shuffled = g_new (int, PL_LEN);
-	playlist->priv->shuffle_len = PL_LEN;
+	array = g_array_sized_new (FALSE, FALSE, sizeof (RandomData), PL_LEN);
 
-	array = g_array_sized_new (FALSE, FALSE,
-			sizeof (RandomData), PL_LEN);
-
-	for (i = 0; i < PL_LEN; i++)
-	{
+	for (i = 0; i < PL_LEN; i++) {
 		data.random = g_random_int_range (0, PL_LEN);
 		data.index = i;
 
@@ -1422,13 +1415,10 @@ ensure_shuffled (TotemPlaylist *playlist, gboolean shuffle)
 
 	g_array_sort (array, compare_random);
 
-	for (i = 0; i < PL_LEN; i++)
-	{
-		playlist->priv->shuffled[i]
-			= g_array_index (array, RandomData, i).index;
+	for (i = 0; i < PL_LEN; i++) {
+		playlist->priv->shuffled[i] = g_array_index (array, RandomData, i).index;
 
-		if (playlist->priv->current != NULL
-				&& playlist->priv->shuffled[i] == current)
+		if (playlist->priv->current != NULL && playlist->priv->shuffled[i] == current)
 			current_new = i;
 	}
 
@@ -1449,7 +1439,13 @@ update_shuffle_cb (GConfClient *client, guint cnxn_id,
 
 	shuffle = gconf_value_get_bool (entry->value);
 	playlist->priv->shuffle = shuffle;
-	ensure_shuffled (playlist, shuffle);
+	if (shuffle == FALSE) {
+		g_free (playlist->priv->shuffled);
+		playlist->priv->shuffled = NULL;
+		playlist->priv->shuffle_len = 0;
+	} else {
+		ensure_shuffled (playlist);
+	}
 
 	g_signal_emit (G_OBJECT (playlist),
 			totem_playlist_table_signals[CHANGED], 0,
@@ -1680,8 +1676,7 @@ totem_playlist_add_one_mrl (TotemPlaylist *playlist, const char *mrl,
 	ref = NULL;
 	uri = totem_create_full_path (mrl);
 
-	D("totem_playlist_add_one_mrl (): %s %s %s\n",
-				filename_for_display, uri ? uri : "(null)", display_name);
+	g_debug ("totem_playlist_add_one_mrl (): %s %s %s\n", filename_for_display, uri, display_name);
 
 	if (playlist->priv->tree_path != NULL && playlist->priv->current != NULL) {
 		int *indices;
@@ -1726,7 +1721,8 @@ totem_playlist_add_one_mrl (TotemPlaylist *playlist, const char *mrl,
 
 	if (playlist->priv->current == NULL && playlist->priv->shuffle == FALSE)
 		playlist->priv->current = gtk_tree_model_get_path (playlist->priv->model, &iter);
-	ensure_shuffled (playlist, playlist->priv->shuffle);
+	if (playlist->priv->shuffle)
+		ensure_shuffled (playlist);
 
 	/* And update current to point to the right file again */
 	if (ref != NULL) {
@@ -1934,7 +1930,8 @@ totem_playlist_clear_with_compare (TotemPlaylist *playlist,
 		}
 
 		playlist->priv->current_shuffled = -1;
-		ensure_shuffled (playlist, playlist->priv->shuffle);
+		if (playlist->priv->shuffle)
+			ensure_shuffled (playlist);
 
 		g_signal_emit (G_OBJECT (playlist),
 				totem_playlist_table_signals[CURRENT_REMOVED],
@@ -1947,7 +1944,8 @@ totem_playlist_clear_with_compare (TotemPlaylist *playlist,
 			gtk_tree_row_reference_free (ref);
 		}
 
-		ensure_shuffled (playlist, playlist->priv->shuffle);
+		if (playlist->priv->shuffle)
+			ensure_shuffled (playlist);
 
 		g_signal_emit (G_OBJECT (playlist),
 				totem_playlist_table_signals[CHANGED], 0,
